//====================================================================================
//  Copyright (C) BAY9, 2016
//====================================================================================
//
// MODULE:
//   pltDel
//
// ----------------------------------------------------------------------------------------
// ----------------------------------------------------------------------------------------

module pltDel(y, y_or, y_ff, y_we, 
              x, x_ir, x_fe, x_re, 
              run1, 
              regBus, regWe, regRe, regWeOut, regReOut, clk1, clk2, reset);

  // --------------------------------------------------------------------------------------
  // Module parameters
  parameter y_w               = 0;
  parameter x_w               = 0;

  parameter run1_w            = 0;

  parameter w                 = 0;

  // --------------------------------------------------------------------------------------
  // Inputs and outputs
  output            [y_w-1:0]  y;
  output                       y_or;
  input                        y_ff;
  output                       y_we;
  assign                       y_we = y_or & ~y_ff;

  input             [x_w-1:0]  x;
  output                       x_ir;
  input                        x_fe;
  output                       x_re;
  assign                       x_re = x_ir & ~x_fe;

  input          [run1_w-1:0]  run1;

  // Define clock and reset
  input                        clk1;
  input                        clk2;
  input                        reset;

  // Inputs and outputs for registers
  inout                [15:0]  regBus;
  input                        regWe;
  input                        regRe;
  output                       regWeOut;
  output                       regReOut;

  // Assign clock
  wire clk;
  assign clk = clk1;

  // --------------------------------------------------------------------------------------
  // External status registers
  // --------------------------------------------------------------------------------------
  // Define register control output
  assign regWeOut = 0;
  assign regReOut = 0;

// ----------------------------------------------------------------------------------------
//=========================================================================================

  // --------------------------------------------------------------------------------------
  // Register definitions
  reg     [5:0]  cnt;         // Data counter
  
  wire           isData;      // As the name says, use for indicating
  wire           isPilot;     // either data or a pilot
  
  // --------------------------------------------------------------------------------------
  // Assign input enable
  assign x_ir  = run1 & (isPilot | ~y_ff);
  
  // Test for special counter values
  assign isPilot = (cnt==0) | (cnt==1) | (cnt==2) | (cnt==3);
  assign isData  = ~isPilot;
  
  // --------------------------------------------------------------------------------------
  // FSM
  always @(posedge clk) begin
    
    // Reset
    if (run1==0) begin
      cnt   <= 0;
      
    // Normal operation
    end else begin
      
      // Input counter FSM
      if (cnt!=51 & x_re) begin
        cnt <= cnt+1;
      end else 
      if (cnt==51 & x_re) begin
        cnt <= 0;
      end
     
    end // if (run1_q==0) ... else
  end // always

  // --------------------------------------------------------------------------------------
  // Assign output data carriers
  assign y    = x;
  assign y_or = isData & x_re;
  
endmodule
//=========================================================================================

